# KPITAINE Tooling
#
# Synchronisation de fichiers avec le Kpitaine Drive via le protocole sFTP
#
# @author it@kpitaine.com
# Pour toute question, consulter la FAQ sur le rubrique d�di�e
#
# @see https://faq.kpitaine.com/sources-de-donn�es/kpitaine-drive/synchronisation-kpitaine-drive
#
# Vous pouvez ais�ment modifier le comportement de la synchronisation
# selon vos souhaits en personnalisant le script � votre guise.
# Plus d'information sur les possibilit�s de l'API WinSCP sur
# https://winscp.net/eng/docs/library
#

. .\config.ps1


if ($SRC_PATH -eq $null -or $DEST_PATH -eq $null -or $SERVER_HOST -eq $null) {
    [Console]::Error.WriteLine("Missing configuration item")
    Break Script
}
if ( ! (Test-Path $SRC_PATH -PathType Container) ) {
    [Console]::Error.WriteLine("$SRC_PATH path is not found")
    Break Script
}


if ( ! (Test-Path $SERVER_CERTFILE -PathType Leaf) ) {
    [Console]::Error.WriteLine("Certificate  file $SERVER_CERTFILE is not found")
    Break Script
}

if ( !($QUIET) ) {
    echo ""
    echo "     | |/ /| _ \|_ _||_   _|/_\  |_ _|| \| || __| "
    echo "     | ' < |  _/ | |   | | / _ \  | | | .\` || _| "
    echo "     |_|\_\|_|  |___|  |_|/_/ \_\|___||_|\_||___| "
    echo ""
    echo "                    KPITAINE DRIVE "
    echo "                     <-- SYNC -->"
    echo ""


    echo " - Copying '$SRC_PATH' source files to Kpitaine sFTP server..."

}

# Copie des fichiers
$fileCount = (Get-ChildItem -Recurse -File $SRC_PATH|Measure-Object).Count


# Utilisation de WinSCP

# Charger l'ensemble .NET de WinSCP
Add-Type -Path "WinSCPnet.dll"

# Configurer les options de session
$sessionOptions = New-Object WinSCP.SessionOptions -Property @{
    Protocol = [WinSCP.Protocol]::Sftp
    HostName = $SERVER_HOST
    PortNumber = $SERVER_PORT
    UserName = $SERVER_USERNAME
    GiveUpSecurityAndAcceptAnySshHostKey = 1
    SshHostKeyFingerprint = "ssh-rsa 8192 wxyUFbdIubQWMcFsDXR1u4bWj+Q/rAhVXBKJ9qZMC2g="
    SshPrivateKeyPath = $SERVER_CERTFILE
}

$sessionOptions.AddRawSettings("AuthKI", "0")

$session = New-Object WinSCP.Session

try
{
    If (!$QUIET)
    {
        echo " - Connecting to server ${SERVER_HOST}:${SERVER_PORT}"
    }
    # Connecter
    $session.Open($sessionOptions)

    If (!$QUIET)
    {
        echo " - Copying $fileCount file(s)..."
    }


    $transferOptions = New-Object WinSCP.TransferOptions
    # Exlude temporary files. i.e. files created while office is opened
    $transferOptions.FileMask = "|~*"

    $synchronizationResult = $session.SynchronizeDirectories(
        [WinSCP.SynchronizationMode]::Remote,
        $SRC_PATH,
        $DEST_PATH,
        $FULL_MIRRORING,
        $FULL_MIRRORING,
        [WinSCP.SynchronizationCriteria]::Time,
        $transferOptions
    )


    If ($synchronizationResult.IsSuccess) {
        If (!$QUIET) {
            echo ""
            echo " [OK] Transfer completed"
        }
    } Else {
        If (!$QUIET) {
            echo ""
        }
        Write-Error -Category PermissionDenied -ErrorId 403 "[KO] Errors during transfer"
    }

    If (!$QUIET)
    {
        echo ""
        $uploads = $synchronizationResult.Uploads.Count
        echo " - $uploads uploaded file(s)";
         foreach ($file in $synchronizationResult.Uploads) {
            $fd = $file.FileName + " [" + $file.Length + " bytes]"
            echo "    > $fd"
        }
        $removals = $synchronizationResult.Removals.Count
        echo " - $removals removed file(s)";
        foreach ($file in $synchronizationResult.Removals) {
            $fd = $file.FileName
            echo "    > $fd"
        }
    }
    $failures = $synchronizationResult.Failures.Count
    if ( $QUIET -eq $false -or $failures -gt 0)
    {
        echo " - $failures file transfer error(s)";
        foreach ($error in $synchronizationResult.Failures) {
            $eMsg = $error.ToString().Split([Environment]::NewLine) | Select -First 1
            echo "    > $eMsg"
        }
        echo ""
    }

}
finally
{
    $session.Dispose()
}
